/*
  RFM 12 Test program, sends out a short signal

 */
#include <stdint.h>
#include <avr/sleep.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/power.h>

#define UPDATE_INTERVALL 150
#define NODE_ID 7
#define SENSOR_SUPPLY 1
#define WARMUP_DELAY 10

#include "rfm12.h"
#include "sensor.h"

volatile uint8_t send_buffer[] = "ggt1t2t3";
volatile uint16_t gap,temp1,temp2,temp3;
uint8_t temp_ddr, temp_port;



ISR(INT0_vect) {
  rfm12_interrupt();
}

void sleepNow(uint8_t delay) {
  rfm12_startSleep(delay);

  set_sleep_mode(SLEEP_MODE_PWR_DOWN);   // sleep mode is set here
  sleep_enable();          // enables the sleep bit in the mcucr register
                           // so sleep is possible. just a safety pin
  sei();

  sleep_mode();            // here the device is actually put to sleep!!
                           // THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP
  cli();
  sleep_disable();         // first thing after waking from sleep:
                           // disable sleep...
}

void startADC(void){
  PRR = ~(1<<PRADC);  //Disable everything except ADC
  ADCSRA |= (1 << ADEN | 1 << ADPS2 | 1 << ADPS1);  //enable ADC, prescaler 64
}

void stopADC(void){
  ADCSRA = ADCSRA & ~(1 << ADEN);  //disable ADC
  PRR = 0;  //Disable everything
}

void setup(void) {
  wdt_disable();
  cli();
  PRR = 0;  //Disable everything
  PORTB = 0xFF; //Pull up on everything
  rfm12_init(NODE_ID, RFM12_868MHZ);

  // Activate interupt on low level
  GIMSK |= ( 1 << INT0);
  MCUCR &=~(1 << ISC00);
  MCUCR &=~(1 << ISC01);
  cli();
}


uint16_t adcRead( uint8_t channel )
{
  while (ADCSRA & (1<<ADSC) ) __asm volatile ("nop"::);
  // Select channel
  ADMUX = (ADMUX & ~(0x0F)) | (channel & 0x0F);

  //Measure
  ADCSRA |= (1<<ADSC);
  while (ADCSRA & (1<<ADSC) ) __asm volatile ("nop"::);
  return ADCW;
}

void readSensors(void ) {
  temp_ddr  = SENSOR_DDR;
  temp_port = SENSOR_PORT;
  startADC();

  SENSOR_VCC_OUT();
  SENSOR_VCC_LOW();
  SENSOR_1_IN();
  SENSOR_1_LOW();
  SENSOR_2_IN();
  SENSOR_2_LOW();

  ADMUX &=~(1 << REFS0 | 1 << REFS2); // 1.1 reference
  ADMUX |= (1 << REFS1);
  temp1 = adcRead(SENSOR_INTERNAL_CHANNEL);
  // discard first measurement
  temp1 = adcRead(SENSOR_INTERNAL_CHANNEL);

  ADMUX &=~(1 << REFS0 | 1 << REFS1 | 1 << REFS2); // VCC reference
  // discard first measurement
  gap = adcRead(SENSOR_GAP_CHANNEL);
  gap = adcRead(SENSOR_GAP_CHANNEL);
  temp2 = 1023 - adcRead(SENSOR_1_CHANNEL);
  temp3 = 1023 - adcRead(SENSOR_2_CHANNEL);

  stopADC();
  SENSOR_PORT = temp_port;
  SENSOR_DDR  = temp_ddr;
}

void loop(void) {
  uint8_t i;

  for( i = 0; i < 8; i++)
    send_buffer[i] = 0;

  readSensors();

  send_buffer[0] = gap   >> 8;
  send_buffer[1] = gap   & 0xFF;
  send_buffer[2] = temp1 >> 8;
  send_buffer[3] = temp1 & 0xFF;
  send_buffer[4] = temp2 >> 8;
  send_buffer[5] = temp2 & 0xFF;
  send_buffer[6] = temp3 >> 8;
  send_buffer[7] = temp3 & 0xFF;

  rfm12_sendBuffer(0, (uint8_t*) send_buffer, 8, RFM12_MESSAGE_TYPE_TEMP_DATA, 0);
  sleepNow(UPDATE_INTERVALL);
}

int main(void) {
  setup();

  while(1) {
    loop();
  }

  return 1;

}